{% extends "base.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Invitation Expired{% endblock %}
{% block pagetitle %}Invitation Expired{% endblock %}

{% block content %}
<p>
  Sorry, your invitation code has expired or is not valid.
</p>
{% endblock %}
