{% extends "base.html" %}
{% load kegweblib crispy_forms_tags %}

{% block title %}Reset Password{% endblock %}

{% block content %}
<h2>E-Mail Sent</h2>
<p>
  We've sent password reset instructions to the address provided.
</p>
{% endblock %}
