{% extends "base.html" %}
{% load kegweblib %}

{% block title %}Confirm Your E-Mail Address{% endblock %}
{% block pagetitle %}Confirm Your E-Mail Address{% endblock %}

{% block content %}
<p>
  You're almost done! Please click the activation link in your e-mail
  to continue.
</p>
{% endblock %}
