{% extends "base.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Invitation Required{% endblock %}
{% block pagetitle %}Invitation Required{% endblock %}

{% block content %}
<p>
  Sorry, you need an invitation in order to register on this system.
</p>
{% endblock %}
