<div class="alert alert-info">
<h4>This Kegbot is running in <em>Demo Mode</em></h4>

<p>
  All features are enabled, but you won't be able to register
  for an account, change system settings, or save any other
  data.
</p>

<p>
  <form action="/demo/summon-drinker/" method="POST" style="margin-bottom: 0px;">
  <button class="btn" type="submit">Summon a Drinker</button>
  </form>
</p>
</div>