{% load kegweblib %}
{% load humanize %}

{% for volume_ml,user in ranked_drinkers %}
{% with forloop.counter as rank %}
<div class="well well-small">
<div class="row-fluid">
  <div class="span3 offset 1">
    {% mugshot_box user 64 %}
  </div>
  <div class="span8">
    <h4>
      {{ user.get_full_name }} <br/>
      <small>
      #{{ rank }} &middot; {% volume volume_ml %}
      </small>
    </h4>
  </div>
</div>
</div>
{% endwith %}
{% endfor %}
